DIVIDER = 'divider'


class PreludeIndexPage(object):
    
    def __init__(self, template_name=None, show_apps=False):
        '''
        Constructor of PreludeIndexPage
        '''
    
        self.template_name = template_name
        self.show_apps = show_apps
    
    
    def get_context(self, request):
        '''
        Data that it's going to be sent to view; You need to overwrite it in order to send your own data.
        
        Parameters:
            request - Request object that it will be injected into this function before the final rendering; (in case to capture session)
        '''
        
        return {}


class PreludeDefaultIndexPage(PreludeIndexPage):
    
    def __init__(self, *args, **kwargs):
        super(PreludeDefaultIndexPage, self).__init__(*args, **kwargs)
        
        self.template_name = 'admin/includes/index_customized.html'
        self.show_apps = True      
        

class PreludeAdminCustomizer(object):
    
    def __init__(self, site_header=None, site_title=None, show_about=True, enable_pwa=True):
        self.site_header = site_header
        self.site_title = site_title
        self.site_logo = None
        
        self.main_menu = []
        
        self.site_css = 'site_default.css'
        self.show_about = show_about

        self.index = None
        self.configure_index()

        self.enable_pwa = enable_pwa

    def configure_site_logo(self, header_logo=None, bigger_logo=None):
        self.site_logo = {
            'header_logo': header_logo,
            'bigger_logo': bigger_logo
        }

    def configure_index(self, index=PreludeIndexPage()):
        self.index = index
        
    def register_menu(self, name, icon=None, to=None, items=None):
        menu_item = {
            'name': name,
            'icon': icon,
            'to': to,
            'items': None
        }
       
        if items is not None: 
            normalized_items = [
                {
                    'name': x.get('name', None),
                    'icon': x.get('icon', None),
                    'to': x.get('to', None),
                    'type': x.get('type', 'item')
                } for x in items
            ]
            
            menu_item.update({'items': normalized_items})
        
        self.main_menu.append(menu_item)

    def get_context_vars(self, request):
        
        return {
            'custom_menu': self.main_menu,
            'admin': {
                'site_css': self.site_css,
                'show_about': self.show_about,
                'enable_pwa': self.enable_pwa,
                'logo': self.site_logo,
                'index': {
                    'template_name': self.index.template_name,
                    'show_apps': self.index.show_apps,
                    'context': self.index.get_context(request)
                }
            }
        }



# Using example inspired on the github below: Thanks!
# https://github.com/nathanbigaignon/django-uikit-admin/blob/master/uikit_admin/templatetags/uikit_admin.py

from django import template
from django.forms.boundfield import BoundField
from django.utils.html import format_html

from django.forms.widgets import Select
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper, AdminTextareaWidget

register = template.Library()


@register.simple_tag
def uka_form_row_stacked(element, errors='', extra_classes=''):
    label = BoundField.label_tag(element, "", {'class': 'uk-form-label'}, label_suffix='')
    
    if errors:
        extra_classes = f'{extra_classes} uk-form-danger uk-clearfix'
    
    help_text = f'<div class="uk-text-muted uk-margin-small-top"><span uk-icon="icon: comment"></span> {element.help_text}</div>' \
        if element.help_text else ''
    
    original_classes = element.field.widget.attrs.get('class', '')
    
    applied_classes = 'uk-margin-small-top uk-margin-small-bottom'
    
    if element.field.widget.__class__.__name__ == "Select":
        applied_classes = f'{original_classes} uk-select'
    
    elif element.field.widget.__class__.__name__ == "RelatedFieldWidgetWrapper":
        applied_classes = f'{original_classes} uk-select'
        
    elif element.field.widget.__class__.__name__ in ["AdminTextareaWidget", "TextareaWidget"]:
        applied_classes = f'{original_classes} uk-textarea'
    else:
        applied_classes = original_classes
    
    
    # Trying some overrides
    if element.field.__class__.__name__ in ['SplitDateTimeField', 'ReadOnlyPasswordHashField', 'ModelMultipleChoiceField']:
        element = element.as_widget()
    
    elif element.field.__class__.__name__ == 'TextareaWidget':
        element = element.as_widget(attrs={'class': f'{applied_classes}'})
   
    else:
        element = element.as_widget(attrs={'class': f'uk-input uk-form-width-large {applied_classes} {extra_classes}'})
    
    html_error = format_html(f'<div class="uk-text-danger uk-margin-top">{errors}</div>')
    
    html = format_html(f'<div class="uk-form-row">' \
                       f'    <div>{label} {html_error}</div>' \
                       f'    <div class="uk-form-controls" style="clear: both">{element}{help_text}</div>' \
                       f'</div>')
    return html

@register.simple_tag
def uk_element (element, css_class_override=None):
    
    if css_class_override is not None:
        element = element.as_widget(attrs={'class': css_class_override })
    else:
        element = element.as_widget()
        
    return format_html(element)
    

@register.simple_tag
def uka_form_row_stacked_button(text, classes=None):  # @todo Fix this
    if classes is None:
        classes = ''
    html = format_html(f'<div class="uk-form-row"><div class="uk-form-controls"><button class="uk-button {classes}">{text}</button></div></div>')
    return html


@register.simple_tag
def uka_button(text, classes=None, type_name=None, name=None):
    if classes is None:
        classes = ''
    if type_name is None:
        type_name = ''
    if name is None:
        name = ''
    html = format_html(
        '<button class="uk-button {}" type="{}" name="{}">{}</button>',
        classes, type, name, text)
    return html


@register.filter(name='addcss')
def addcss(field, css):
    return field.as_widget(attrs={"class": css})


from django import template


register = template.Library()

@register.simple_tag
def full_url_without_tags(request, *args):
    """Removes all values of arg from the given string"""

    full_path = request.get_full_path()
    query_part = full_path.split('?')
    
    if len(query_part) == 1:
        return full_path
     
    tags = [ x.split('=') for x in query_part[1].split('&')]
    
    tags_final = []
    for item in tags:
        if item[0] not in args:
            tags_final.append('='.join(item))
    
    
    full_path = f'{query_part[0]}?{"&".join(tags_final)}'
    
    return full_path



from django import forms
# from base.widgets import ClearableUUIDSecureFileInput
# from django_select2.forms import Select2Widget


class PrlModelForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):

        super(PrlModelForm, self).__init__(*args, **kwargs)

        # Overriding File Field
        print(self.fields)

        for key in self.fields.keys():
            field_type = type(self.fields[key]).__name__

            # print(f'FIELD TYPE: {field_type}')

            # if field_type == 'CharField':
            widget_type = type(self.fields[key].widget).__name__

            if widget_type == 'AdminTextareaWidget':
                self.fields[key].widget.attrs['class'] = \
                    self.fields[key].widget.attrs['class'] + ' uk-textarea'
            elif widget_type == 'AdminTextInputWidget':
                self.fields[key].widget.attrs['class'] = \
                    self.fields[key].widget.attrs['class'] + ' uk-input'

            #self.fields[key].widget.attrs['class'] = self.fields[key].widget.attrs['class'] + ' uk-input'

            #if field_type == 'FileField':
            #    self.fields[key].widget = ClearableUUIDSecureFileInput()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['prelude_django_admin_toolkit',
 'prelude_django_admin_toolkit.migrations',
 'prelude_django_admin_toolkit.templatetags']

package_data = \
{'': ['*'],
 'prelude_django_admin_toolkit': ['locale/pt_BR/LC_MESSAGES/*',
                                  'static/*',
                                  'static/prelude_django_admin_toolkit/*',
                                  'static/prelude_django_admin_toolkit/css/*',
                                  'static/prelude_django_admin_toolkit/icon/*',
                                  'static/prelude_django_admin_toolkit/js/*',
                                  'templates/admin/*',
                                  'templates/admin/auth/user/*',
                                  'templates/admin/edit_inline/*',
                                  'templates/admin/includes/*',
                                  'templates/admin/pages/*',
                                  'templates/admin/widgets/*',
                                  'templates/registration/*']}

install_requires = \
['django>=3,<4']

setup_kwargs = {
    'name': 'prelude-django-admin-toolkit',
    'version': '0.14.3',
    'description': 'An alternative Django Admin theme with Batteries; project Bruh',
    'long_description': None,
    'author': 'Jonhnatha Trigueiro',
    'author_email': 'joepreludian@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from django.contrib.admin.apps import SimpleAdminConfig


class PrlAdminConfig(SimpleAdminConfig):
    """The default AppConfig for admin which does autodiscovery."""

    default_site = 'prelude_django_admin_toolkit.admin.PrlAdmin'

    def ready(self):
        super().ready()
        self.module.autodiscover()



from django.contrib import admin
from django.conf import settings

from prelude_django_admin_toolkit.customizer import PreludeAdminCustomizer
from django.shortcuts import render

'''
class PrlActionForm(helpers.ActionForm):
    action = forms.ChoiceField()
    action.widget.attrs.update({
        'class': 'uk-select uk-width-1-1 uk-padding-remove-right'
    }) 
'''


class PrlModelAdmin(admin.ModelAdmin):
    save_as = True
    list_per_page = 50


class PrlAdmin(admin.AdminSite):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # Custom settings Dict
        custom_settings = getattr(settings, 'PRELUDE_ADMIN', None)

        # If you haven't set a custom settings, create one
        if custom_settings is None:
            custom_settings = {
                'customizer_object': PreludeAdminCustomizer()
            }

        self.prl_customizer = custom_settings['customizer_object']
        self.load_customizer()

    def load_customizer(self):
        if self.prl_customizer.site_header:
            self.site_header = self.prl_customizer.site_header

        if self.prl_customizer.site_title:
            self.site_title = self.prl_customizer.site_title

        self.show_about = self.prl_customizer.show_about

    def each_context(self, request):
        context_vars = super().each_context(request)

        context_vars.update(self.prl_customizer.get_context_vars(request))

        return context_vars

    def page_about(self, request):
        return render(request, 'admin/pages/about.html', self.each_context(request))

    def get_urls(self):
        from django.urls import path

        url_patterns = super().get_urls()

        if self.show_about:
            url_patterns.append(
                path('about/', self.admin_view(self.page_about), name='about')
            )

        return url_patterns

    '''
    @never_cache
    def index(self, request, extra_context=None):
        
        context = {'name': 'JON123456'}
        
        return admin.AdminSite.index(self, request, extra_context=context)
    '''


site = PrlAdmin()

from django.contrib.auth.admin import UserAdmin, GroupAdmin
from django.contrib.auth.models import User, Group

site.register(User, UserAdmin)
site.register(Group, GroupAdmin)
